package com.bweb.controller;

import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.hfkj.common.exception.ErrorCode;
import com.hfkj.common.exception.ErrorHelp;
import com.hfkj.common.exception.SysCode;
import com.hfkj.common.utils.ResponseMsgUtil;
import com.hfkj.model.ResponseData;
import com.hfkj.model.UserSessionObject;
import com.hfkj.service.gas.BsGasOrderReceiptService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Controller
@RequestMapping(value = "/gasOrderReceipt")
@Api(value = "加油订单业务")
public class BsGasOrderReceiptController {
    private static Logger log = LoggerFactory.getLogger(BsGasOrderReceiptController.class);
    @Resource
    private BsGasOrderReceiptService gasOrderReceiptService;

    @RequestMapping(value="/success",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "开票成功")
    public ResponseData success(@RequestBody JSONObject body) {
        try {
            if (body == null
                || StringUtils.isBlank(body.getString("applyNo"))) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            gasOrderReceiptService.success(body.getString("applyNo"));

            return ResponseMsgUtil.success("申请成功");

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/fail",method = RequestMethod.POST)
    @ResponseBody
    @ApiOperation(value = "开票失败")
    public ResponseData fail(@RequestBody JSONObject body) {
        try {
            if (body == null
                    || StringUtils.isBlank(body.getString("applyNo"))
                    || StringUtils.isBlank(body.getString("remark"))
            ) {
                throw ErrorHelp.genException(SysCode.System, ErrorCode.REQ_PARAMS_ERROR, "");
            }

            gasOrderReceiptService.fail(body.getString("applyNo"),body.getString("remark"));

            return ResponseMsgUtil.success("申请成功");

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }

    @RequestMapping(value="/queryReceiptList",method = RequestMethod.GET)
    @ResponseBody
    @ApiOperation(value = "查询申请列表")
    public ResponseData queryReceiptList(@RequestParam(name = "orderNo", required = false) String orderNo,
                                       @RequestParam(name = "contactsPhone", required = false) String contactsPhone,
                                       @RequestParam(name = "status", required = false) Integer status,
                                       @RequestParam(name = "createTimeS", required = false) Long createTimeS,
                                       @RequestParam(name = "createTimeE", required = false) Long createTimeE,
                                       @RequestParam(name = "pageNum", required = true) Integer pageNum,
                                       @RequestParam(name = "pageSize", required = true) Integer pageSize) {
        try {
            Map<String,Object> param = new HashMap<>();
            param.put("orderNo", orderNo);
            param.put("contactsPhone", contactsPhone);
            param.put("status", status);
            param.put("createTimeS", createTimeS);
            param.put("createTimeE", createTimeE);

            PageHelper.startPage(pageNum,pageSize);
            return ResponseMsgUtil.success(new PageInfo<>(gasOrderReceiptService.getList(param)));

        } catch (Exception e) {
            log.error("error!",e);
            return ResponseMsgUtil.exception(e);
        }
    }


}
